package Elements;
/**
 * Air describes the humid air psychrometric properties used in the LDDS Simulation.
 * 
 * @see FMix
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class Air extends Fluid{

	/**
	 * Creates an instance of Air with 25.0 C of drybulb temperature, 101.325 kPa 
	 * of atmospheric pressure and 20 g/kg of humidity ratio.
	 * 
	 */
	public Air(){
		a_name = "Air";
		try{
		a_Tem = 25.0;
		a_Pressure = 101.325;
		a_XCon = Air.getXConForW(0.01);
		refresh();
		} catch (OutOfRangeException ex){
			System.out.println(ex.getMessage());
		}
	}
	
	/**
	 * Creates an instance of Air with specified parameters.
	 * 
	 * @param DBT a <code>double</code> value for drybulb temperature (C)
	 * @param Pa a <code>double</code> value for atmospheric pressure (kPa)
	 * @param Wa a <code>double</code> value for humidity ratio (kg/kg)
	 * 
	 */
	public Air(double DBT, double Pa, double Wa) throws OutOfRangeException{
		a_name = "Air";
		a_Tem = DBT;
		a_Pressure = Pa;
		a_XCon = Air.getXConForW(Wa);
		refresh();
	}
	
	/**
	 * Specifies the concentration of water of Air
	 * 
	 * @param xcon a <code>double</code> value of concentration of water (kg/kg)
	 * 
	 */
	@Override
	public void setXCon(double v_XCon) throws OutOfRangeException{
		a_XCon = v_XCon;
		refresh();
	}
	
	/**
	 * Specifies the humidity ratio of Air
	 * 
	 * @param Wa a <code>double</code> value of humidity ratio (kg/kg)
	 * 
	 */
	@Override
	public void setHum() throws OutOfRangeException{
		a_Hum = a_XCon / (1 - a_XCon);
		if (getRH() >= 1.0001){
			throw new OutOfRangeException();
		}
	}
		
	
	public double getSHum(){
		double W, Ws;
		W = Ws = getHum();
		try{
		setRH(1.0);
		Ws = getHum();
		setXCon(Air.getXConForW(W));
		} catch (OutOfRangeException ex){
			//No reached
		}
		return Ws;
	}
	
	
	/**
	 * It calculates the partial saturated pressure of water for specified
	 * temperature
	 */
	@Override
	protected void setPs(){
		double T;
		double[] C = {-5.8002206e3, 1.3914993, -4.8640239e-2, 4.1764768e-5,
				-1.4452093e-8, 6.5459673};
		T = getTem() + 273.15;
		a_Ps = Math.exp(C[0] / T + C[1] + C[2] * T + C[3] * Math.pow(T,2) 
				+ C[4] * Math.pow(T, 3) + C[5] * Math.log(T)) / 1000;
	}
	
	/**
	 * It calculates the specific density of Air (kg/m^3)
	 */
	@Override
	protected void setDensity(){
		double T;
		T = getTem() + 273.15;
		a_Density = getPressure() /(0.287042 * T * (1 + 1.607858 * a_XCon));
	}
	
	/**
	 * It calculates the specific enthalpy of Air (kJ/kg)
	 */
	@Override
	protected void setSpEnthalpy(){
		a_SpEnthalpy = 1.007 * getTem() + a_XCon * (2501 + 1.86 * getTem());
	}
	
	/**
	 * It calculates and adjusts the specific heat of Air (kJ/kg-K)
	 */
	@Override
	protected void setSpHeat(){
		a_SpHeat = 1.007 + 1.86 * getXCon();
	}
	
	public boolean setRH(double RH){
		double backup = a_Hum;
		if (RH < 0.0 || RH > 1.00){
			return false;
		} else {
			try{
			double pw = RH * getPs();
			a_Hum = 0.621945 / (getPressure() / pw - 1.0);
			setXCon(Air.getXConForW(a_Hum));
			return true;
			} catch (OutOfRangeException ex){
				a_Hum = backup;
				return false;
			}
		}
	}
	
	public double getRH(){
		double pw = getPressure() / (0.621945 / getHum() + 1.0);
		return pw / getPs();  
	}
	
	public static double getXConForW(double wa){
		return wa / (1 + wa);
	}
	
	@Override
	protected void verify() throws OutOfRangeException{
		if(a_Pressure < 0.0 || a_Tem < -273.15 || getRH() > 1.0001){
		throw new OutOfRangeException();
		}
	}

}
