package Elements;
/**
 * Desiccant describes the physical properties of an aqueous solution.
 * 
 * @see FMix
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class Desiccant extends Fluid{
	private Salt a_type;
	
	/**
	 * Salt types that can be used
	 */
	enum Salt{
		/**
		 * Lithium Chloride
		 */
		LiCl,
		/**
		 * Calcium Chloride
		 */
		CaCl2
	}
	
	/**
	 * Creates an instance with 20.0 C of drybulb temperature, 101.325 kPa of
	 * atmospheric pressure and 0.5 kg/kg of Calcium Chloride salt concentration.
	 * 
	 * @see Fluid
	 */
	public Desiccant(){
		super();
		try{
			a_name = "Desiccant";
			setType(Salt.CaCl2);
			setXCon(0.3);
		} catch (OutOfRangeException ex){
			System.out.println(ex.getMessage());
			System.out.println("At: Desiccant Constructor");
		}
	}
	
	/**
	 * Creates an new instance with the specified salt type, with 20.0 C of drybulb
	 * temperature, 101.325 kPa
	 * 
	 * @param type an <code>integer</code> value of Salt enumeration
	 * @throws OutOfRangeException 
	 */
	public Desiccant(Salt type) throws OutOfRangeException{
		super();
		a_name = "Desiccant";
		setType(type);
		setXCon(0.3);
		setPressure(101.325);
		verify();
	}
	
	/**
	 * Adjust the type of salt based of the enumerated internal database
	 * 
	 * @param type Integer value from enumerated type of salts
	 */
	public void setType(Salt type) throws OutOfRangeException{
		a_type = type;
		refresh();
	}
	
	/**
	 * Returns the integer value of enumerated type of salt
	 * 
	 * @return Integer value of enumerated type of salt
	 */
	public Salt getType(){
		return a_type;
	}
	
	/**
	 * 
	 */
	public double getSolBoundary(){
		double th;
		int region = 0;
		double[] a = {0.253, 0.287, 0.369, 0.452, 0.558};
		double[][] A = {{0.422088, -0.09041, -2.936350}, {-0.00534, 2.01589, -3.11459},
				{-0.56036, 4.72308, -5.81105}, {-0.31522, 2.88248, -2.62433},
				{-1.31231, 6.17767, -5.03479}, {-1.356800, 3.448540, 0.0}};
		double[] b = {0.2985, 0.4984, 0.5663, 0.7486};
		double[][] B = {{0.422088, -0.066933, -0.282395, -355.514247},
				{-0.37895, 3.4569, -3.53131, 0.0}, {-0.51997, 3.40097, -2.85129, 0.0},
				{-2.385836, 8.084829, -5.303476, 0.0}, {-2.80756, 4.67825, 0.0, 0.0}};
		if (a_type == Salt.LiCl){
			for(int i = 0; i < a.length; i++){
				region = (a_XCon > a[i]) ? i + 1 : region;
			}
			if (region == 0){
				th = A[0][0] + A[0][1] * a_XCon + A[0][2] * Math.pow(a_XCon, 2.5);
			} else {
				th = A[region][0] + A[region][1] * a_XCon + A[region][2] * a_XCon * a_XCon;
			}
		} else {
			for(int i = 0; i < b.length; i++){
				region = (a_XCon > b[i]) ? i + 1 : region;
			}
			if (region == 0){
				th = B[0][0] + B[0][1] * a_XCon + B[0][2] * a_XCon * a_XCon 
						+ B[0][3] * Math.pow(a_XCon, 7.5);
			} else {
				th = B[region][0] + B[region][1] * a_XCon + B[region][2] * a_XCon * a_XCon;
			}
		}
		return th * 647.14 - 273.15;
	}
	
	/**
	 * Interpolates to find the closest concentration to obtain the desired humidity
	 * ratio
	 * 
	 * @param hum <code>double</code> value of humidity ratio
	 */
	public void setHum(double hum) throws OutOfRangeException{
		setXCon(0.0);
		if (getHum() < hum) {
			throw new OutOfRangeException();
		}
		boolean flag = true;
		setXCon(0.00001);
		while(flag){
			flag = (getHum() < hum) ? false: true;
			setXCon(getXCon() + 0.00001);
		}
		verify();
		refresh();
	}
	
	/**
	 * Equilibrium humidity ratio
	 * 
	 * @return W <code>double</code> value of equilibrium humidity ratio (kg/kg)
	 */
	@Override
	public double getHum(){
		// Con la presin de saturacin hallar la humedad equivalente del aire
		double W = 0.621945 * getPs() / (getPressure() - getPs());
		return W;
	}
	
	/**
	 * It calculates and adjusts the specific heat based on the specified pressure,
	 * temperature and concentration
	 */
	protected void setSpHeat(){
		try{
		Water w = new Water();
		w.setTem(getTem());
		double[][] a = {{1.43980, -1.24317, -0.12070, 0.12825, 0.62934},
				{1.63799, -1.69002, 1.05124, 0.0, 0.0}};
		double[] b = {58.5225, -105.6343, 47.7948};
		double f1,f2,tau;
		tau = (getTem() + 273.15) / 228.0 - 1.0;
		if (getType() == Salt.LiCl){
			if (a_XCon <= 0.31){
				f1 = a[0][0] * a_XCon + a[0][1] * Math.pow(a_XCon, 2.0) 
						+ a[0][2] * Math.pow(a_XCon,  3.0);
			} else {
				f1 = a[0][3] + a[0][4] * a_XCon;
			}
		} else {
			f1 = a[1][0] * a_XCon + a[1][1] * Math.pow(a_XCon, 2.0) 
					+ a[1][2] * Math.pow(a_XCon,  3.0);
		}
		f2 = b[0] * Math.pow(tau,0.02) + b[1] * Math.pow(tau, 0.04) 
				+ b[2] * Math.pow(tau, 0.06);
		a_SpHeat = w.getSpHeat()* (1 - f1 * f2);
		} catch (OutOfRangeException ex){
			System.out.println(ex.getMessage());
			System.out.println("At: Desiccant.setSpHeat()");
		}
	}
	
	/**
	 * It calculates the density of Desiccant based on Conde equations
	 */
	protected void setDensity(){
		try{
		int index;
		double value;
		Water water = new Water();
		water.setTem(getTem());
		double[][] rho = {{1.0, 0.540966, -0.303792, 0.100791},
						  {1.0, 0.836034, -0.4363, 0.105642}};
		if (getType() == Salt.LiCl){
			index = 0;
		} else{
			index = 1;
		}
		value = 0.0;
		for(int i = 0; i <= 3; i++){
			value += rho[index][i] * Math.pow(getXCon() / (1.0 - getXCon()), (double) i);
		}
		a_Density = water.getDensity() * value;
		} catch(OutOfRangeException ex){
			System.out.println(ex.getMessage());
			System.out.println("At: Desiccant.setDensity()");
		}
	}
	
	/**
	 * It calculates the viscosity using Conde equations
	 */
	@Override
	protected void setViscosity(){
		try{
		double th = (getTem() + 273.15) / 647.14;
		Water w = new Water();
		double[][] n = {{0.0, 0.090481, 1.390262, 0.675875, - 0.583517},
						{0.0, -0.169310, 0.817350, 0.574230, 0.398750}};
		int index = (getType() == Salt.LiCl) ? 0 : 1;
		w.setTem(getTem());
		double E =  getXCon() / Math.pow(1.0 - getXCon(), 1.0 / 0.6);
		a_Viscosity = w.getViscosity() * Math.exp(n[index][1] * Math.pow(E, 3.6) 
						+ n[index][2] * E + n[index][3] * E / th + n[index][4] * E * E);
		} catch(OutOfRangeException ex) {
			System.out.println(ex.getMessage());
			System.out.println("At: Desiccant.setViscosity");
		}
	}
	
	/**
	 * It calculates and adjusts thermal conductivity of solution based on Conde
	 * equations
	 */
	@Override
	protected void setThConductivity(){
		try{
		double[] M = {42.394, 110.98};
		double[] I = {1.0, 2.0};
		double[][] alpha = {{10.8958E-3, -11.7882E-3}, {5.9473E-3, -1.3988E-3}};
		Water w = new Water();
		w.setTem(getTem());
		int index = (getType() == Salt.LiCl) ? 0 : 1;
		double ep = getXCon() * getDensity() * I[index] / M[index];
		double aR = alpha[index][0] + alpha[index][1] * getXCon();
		a_kConduct = w.getThConductivity() - aR * ep;
		} catch(OutOfRangeException ex){
			System.out.println(ex.getMessage());
			System.out.println("At: Desiccant.setThConductivity()");
		}
	}
	
	@Override
	protected void setHfg(){
		double dh0, zt;
		Water w = new Water();
		double[][] H = {{0.0, 0.845, -1.965, -2.265, 0.6, 169.105, 457.850}, {0.0, 0.855, -1.965, -2.265, 0.8, -955.69, 3011.974}};
		int n = (getType() == Salt.LiCl) ? 0 : 1;
		zt = getXCon() / (H[n][4] - getXCon());
		dh0 = H[n][5] + H[n][6] * ((getTem() + 273.15) / 228.0 - 1.0);
		a_SpEnthalpy = w.getHfg() + dh0 * Math.pow(1 + Math.pow(zt / H[n][1], H[n][2]), H[n][3]); 
	}
	
	/**
	 * It calculates and adjusts the saturated pressure (kPa) based on the specified
	 * temperature, pressure and concentration
	 */
	@Override
	protected void setPs(){
		try{
		Water W = new Water();
		W.setTem(getTem());
		double pi[][] = {{0.28,4.3,0.6,0.21,5.1,0.49,0.362,-4.75,-0.4,0.03},
				{0.31,3.698,0.6,0.231,4.584,0.49,0.478,-5.2,-0.4,0.018}};
		double ps_A, ps_B, ps_pi, f1;
		if (getType() == Salt.LiCl){
			ps_A = 2.0 - Math.pow(1.0 + Math.pow(getXCon()/pi[0][0],pi[0][1]),pi[0][2]);
			ps_B = Math.pow(1.0 + Math.pow(getXCon()/pi[0][3], pi[0][4]), pi[0][5]) - 1.0;
			ps_pi = 1.0 - Math.pow(1.0 + Math.pow(getXCon()/pi[0][6],pi[0][7]),pi[0][8])
					- pi[0][9] * Math.exp(-Math.pow(getXCon()-0.1,2.0)/0.005);
		} else {
			ps_A = 2.0 - Math.pow(1.0 + Math.pow(getXCon()/pi[1][0],pi[1][1]),pi[1][2]);
			ps_B = Math.pow(1.0 + Math.pow(getXCon()/pi[1][3], pi[1][4]), pi[1][5]) - 1.0;
			ps_pi = 1.0 - Math.pow(1.0 + Math.pow(getXCon()/pi[1][6],pi[1][7]),pi[1][8])
					- pi[1][9] * Math.exp(-Math.pow(getXCon()-0.1,2.0)/0.005);
		}
		f1 = ps_A + ps_B * Water.getTH(getTem());
		a_Ps = W.getPs() * ps_pi * f1;
		} catch(OutOfRangeException ex){
			System.out.println(ex.getMessage());
			System.out.println("At: Desiccant.setPs()");
		}
	}
	
	/**
	 * Inspect if the temperature is in the correct range for calculations
	 * 
	 * @throws OutOfRangeException 
	 * 
	 */
	public void verify() throws OutOfRangeException{
		if (a_Tem < getSolBoundary()){
			throw new OutOfRangeException();
		}
	}
}
