package Elements;
/**
 * Fluid describes the different thermodynamic properties of a fluid according as its
 * temperature and pressure.
 * 
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class Fluid {
	protected String a_name;
	protected double a_Pressure;
	protected double a_Tem;
	protected double a_XCon;
	protected double a_Hum;
	protected double a_SpHeat;
	protected double a_SpEnthalpy;
	protected double a_SpEntropy;
	protected double a_kConduct;
	protected double a_Density;
	protected double a_Viscosity;
	protected double a_Ps;
	protected double a_Hfg;
	
	/**
	 * It builds a Fluid instance under 101.25 kPa of atmospheric pressure and
	 * 20.0C of ambient drybulb temperature
	 */
	public Fluid(){
		a_name = "Fluid";
		a_Pressure = 101.325;
		a_Tem = 20.0;
		a_XCon = 0.0;
	}
	
	/**
	 * Builds an instance of Fluid with the specified atmospheric pressure (kPa)
	 * and ambient drybulb temperature (C).
	 * 
	 * @param v_Pressure Atmospheric pressure double value (kPa)
	 * @param v_Temp Ambient drybulb temperature double vaule (C)
	 */
	public Fluid(double v_Pressure, double v_Temp)  throws OutOfRangeException{
		a_name = "Fluid";
		setPressure(v_Pressure);
		setTem(v_Temp);
	}
	
	/**
	 * Name of fluid
	 * 
	 * @return Name of the object String value
	 */
	public String getName(){
		return a_name;
	}
	
	/**
	 * Atmospheric Pressure (kPa)
	 * 
	 * @param v_Pressure Atmospheric pressure double value (kPa)
	 */
	public void setPressure(double v_Pressure) throws OutOfRangeException{
		if(v_Pressure < 0.0){
			throw new OutOfRangeException();
		}
		a_Pressure = v_Pressure;
		refresh();
	}
	
	
	/**
	 * It returns atmospheric pressure
	 * 
	 * @return Atmospheric pressure double value (kPa)
	 */
	public double getPressure(){
		return a_Pressure;
	}
	
	/**
	 * Atmospheric drybulb temperature
	 * 
	 * @param v_Temp atmospheric drybulb temperature double value (C)
	 * @throws OutOfRangeException The double value is below -273.15C
	 */
	public boolean setTem(double v_Temp) throws OutOfRangeException{
		if (v_Temp < -273.15){
			return false;
		}
		a_Tem = v_Temp;
		refresh();
		return true;
	}
	
	/**
	 * It returns atmospheric drybulb temperature
	 * 
	 * @return Atmospheric drybulb temperature double value (C)
	 */
	public double getTem(){
		return a_Tem;
	}
	
	/**
	 * Adjust the solute concentration in the mixing (kg_solute/kg_solution)
	 * 
	 * @param v_XCon Concentration double value (kg_solute/kg_solution)
	 * @throws OutOfRangeException The double value is less than 0.0 or more than 1.0
	 */
	public void setXCon(double v_XCon)  throws OutOfRangeException{
		if (v_XCon < 0.0 || v_XCon > 1.0) {
			throw new OutOfRangeException();
		}
		a_XCon = v_XCon;
		refresh();
	}
	
	/**
	 * Solute concentration mixing
	 * 
	 * @return a_XCon Concentration double value (kg/kg)
	 * 
	 */
	public double getXCon(){
		return a_XCon;
	}
	
	/**
	 * Adjust the equilibrium humidity ratio (kg_w/kg_da)
	 * 
	 * @param hum Equilibrium humidity ratio double value(kg_w/kg_da)
	 * @throws OutOfRangeException The value is out of allowable range.
	 */
	protected void setHum() throws OutOfRangeException{
		
	}
	
	/**
	 * Equilibrium humidity ratio (kg_w/kg_da)
	 * 
	 * @return Equilibrium humidity ratio double value (kg_w/kg_da)
	 */
	public double getHum(){
		return a_Hum;
	}
	
	/**
	 * It calculates and adjust the specific heat (kJ/kg-K) based on the specified
	 * pressure an temperature
	 */
	protected void setSpHeat(){
		
	}
	
	/**
	 * Specific Heat (kJ/kg-K)
	 * 
	 * @return Specific heat double value (kJ/kg-K)
	 */
	public double getSpHeat(){
		return a_SpHeat;
	}
	/**
	 * It calculates and adjusts the specific enthalpy (kJ/kg) based on the specified
	 * pressure and temperature
	 */
	protected void setSpEnthalpy(){
		
	}
	
	/**
	 * Specific Enthalpy (kJ/kg)
	 * 
	 * @return Specific enthalpy double value (kJ/kg)
	 */
	public double getSpEnthalpy(){
		return a_SpEnthalpy;
	}
	
	/**
	 * It calculates and adjusts the specific entropy based on the specified pressure
	 * and temperature
	 */
	protected void setSpEntropy(){
		
	}
	
	/**
	 * Specific Entropy (kJ/kg-K)
	 * 
	 * @return Specific entropy double value (kJ/kg-K)
	 */
	public double getSpEntropy(){
		return a_SpEntropy;
	}
	
	/**
	 * It calculates and adjusts the thermal conductivity (W/m-K) based on the specified
	 * pressure and temperature
	 */
	protected void setThConductivity(){
		
	}
	
	/**
	 * Thermal Conductivity (W/m-K)
	 * 
	 * @return Thermal conductivity double value(W/m-K)
	 */
	public double getThConductivity(){
		return a_kConduct;
	}
	
	/**
	 * It calculates and adjust the density (kg/m^3) based on the specified pressure
	 * and temperature
	 */
	protected void setDensity(){
		
	}
	
	/**
	 * Density (kg/m^3)
	 * 
	 * @return Density double value (kg/m^3)
	 */
	public double getDensity(){
		return a_Density;
	}
	
	/**
	 * It calculates and adjusts the dinamic viscosity (Pa-s) based on the specified pressure and
	 * temperature
	 */
	protected void setViscosity(){
		
	}
	
	/**
	 * Dynamic viscosity of fluid (uPa-s)
	 * 
	 * @return <code>double</code> value of dynamic viscosity (uPa-s)
	 */
	public double getViscosity(){
		return a_Viscosity;
	}
	
	/**
	 * Calculates and adjusts the saturated pressure (kPa) based on the specified temperature
	 */
	protected void setPs(){
		
	}
	
	/**
	 * Saturated pressure of fluid (kPa)
	 * 
	 * @return a_Ps Saturated pressure double value (kPa)
	 * 
	 */
	public double getPs(){
		return a_Ps;
	}
	
	/**
	 * It calculates and adjusts the latent heat (kJ/kg) based on the specified temperature
	 */
	protected void setHfg(){
		
	}
	
	/**
	 * Latent heat of fluid
	 * 
	 * @return Latent heat of fluid (kJ/kg)
	 */
	public double getHfg(){
		return a_Hfg;
	}
	
	protected void verify() throws OutOfRangeException{
		if(a_Pressure < 0.0 || a_Tem < -273.15){
		throw new OutOfRangeException();
		}
	}
	
	/**
	 * Update the all the properties when the temperature or pressure are changed
	 */
	protected void refresh() throws OutOfRangeException{
		setPs();
		setHum();
		setDensity();
		setSpHeat();
		setSpEnthalpy();
		setSpEntropy();
		setThConductivity();
		setViscosity();
		verify();
	}
}
