package Elements;
/**
 * HX calculates the outside properties of two fluids that exchange heat and mass
 * based on specified or calculated effectiveness.
 * 
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class HX {
	protected Node DSI;
	protected Node DSO;
	protected Node SSI;
	protected Node SSO;
	protected double EffS;
	protected double EffL;
	protected double UA;
	protected double DPL;
	protected double SPL;
	protected boolean fixed;
	

	/**
	 * Builds an HX instance with sensible effectiveness of 1.0, and fixed latent
	 * effectiveness of 1.0.
	 */
	public HX(){
		DSI = new Node();
		DSO = new Node();
		SSI = new Node();
		SSO = new Node();
		DPL = 0.300;
		SPL = 0.300;
		EffS = 1.0;
		EffL = 1.0;
		fixed = true;
	}
	
	/**
	 * Adjust the Demand Inlet Node
	 * 
	 * @param node Demand Inlet Node
	 * 
	 * @see Node
	 */
	public void setDI(Node node) throws OutOfRangeException{
		DSI.setFluid(node.getFluid());
		DSI.setPressure(node.getPressure());
		DSI.setTem(node.getTem());
		DSI.setXCon(node.getXCon());
		DSI.setMFlow(node.getMFlow());
		refresh();
	}
	
	/**
	 * Return the Demand Inlet Node
	 * 
	 * @return Demand Inlet Node
	 * 
	 * @see Node
	 */
	public Node getDI(){
		return DSI;
	}
	
	/**
	 * Adjust the Supply Inlet Node
	 * 
	 * @param node Supply Inlet Node
	 * 
	 * @see Node
	 */
	public void setSI(Node node) throws OutOfRangeException{
		SSI.setFluid(node.getFluid());
		SSI.setPressure(node.getPressure());
		SSI.setTem(node.getTem());
		SSI.setXCon(node.getXCon());
		SSI.setMFlow(node.getMFlow());
		refresh();
	}
	
	/**
	 * Supply Inlet Node
	 * 
	 * @return Suppy Inlet Node
	 * 
	 * @see Node
	 */
	public Node getSI(){
		return SSI;
	}
	
	/**
	 * Demand Outlet Node
	 * 
	 * @return Demand Outlet Node
	 * 
	 * @see Node
	 */
	public Node getDO(){
		return DSO;
	}
	
	/**
	 * Supply Outlet Node
	 * 
	 * @return Supply Outlet Node
	 * 
	 * @see Node
	 */
	public Node getSO(){
		return SSO;
	}
	
	public boolean setDPL(double loss){
		if(loss >= 0.0){
			DPL = loss;
			return true;
		} else {  
			return false;
		}
	}
	
	public double getDPL(){
		return DPL;
	}
	
	public boolean setSPL(double loss){
		if(loss >= 0.0){
			SPL = loss;
			return true;
		} else {
			return false;
		}
	}
	
	public double getSPL(){
		return SPL;
	}
	
	/**
	 * Adjust a fixed sensible effectiveness.
	 * 
	 * @param eff Sensible effectiveness double value (0.0 - 1.0)
	 */
	public void setEffS(double eff){
		EffS = eff;
		fixed = true;
		refresh();
	}
	
	/**
	 * Sensible effectiveness
	 * 
	 * @return Sensible effectiveness double value (0.0 - 1.0)
	 */
	public double getEffS(){
		return EffS;
	}
	
	/**
	 * Adjust a fixed value of latent effectiveness
	 * 
	 * @param eff Latent effectiveness double value (0.0 - 1.0)
	 */
	public void setEffL(double eff){
		EffL = eff;
		fixed = true;
		refresh();
	}
	
	/**
	 * Latent effectiveness
	 * 
	 * @return Latent effectiveness double value (0.0 - 1.0)
	 */
	public double getEffL(){
		return EffL;
	}
	
	/**
	 * Adjust the Enthalpy Exchanger UA value.
	 * 
	 * @param val Latent effectiveness double value W/K.
	 */
	public void setUA(double val){
		fixed = false;
		UA = val;
		refresh();
	}
	
	public double getUA(){
		return UA;
	}
	
	public boolean isFixed(){
		return fixed;
	}
	
	/**
	 * Update all properties when inlet nodes are changed
	 */
	protected void refresh(){
		DSO.setFluid(DSI.getFluid());
		SSO.setFluid(SSI.getFluid());
		DSO.setPressure(DSI.getPressure() - DPL);
		SSO.setPressure(SSI.getPressure() - SPL);
		double C_d = DSI.getMFlow() * DSI.getCp();
		double C_s = SSI.getMFlow() * SSI.getCp();
		double C_min = Math.min(C_d, C_s);
		double Mf_min = Math.min(SSI.getMFlow(), DSI.getMFlow());
		SSO.setTem(SSI.getTem() 
				- C_min / C_s * EffS * (SSI.getTem() - DSI.getTem()));
		DSO.setTem(DSI.getTem()
				+ C_min / C_d * EffS * (SSI.getTem() - DSI.getTem()));
		SSO.setXCon(SSI.getXCon() - EffL * Mf_min / SSI.getMFlow() * (SSI.getXCon() - DSI.getXCon()));
		DSO.setXCon(DSI.getXCon() + EffL * Mf_min / DSI.getMFlow() * (SSI.getXCon() - DSI.getXCon()));
		DSO.setMFlow(DSI.getMFlow());
		SSO.setMFlow(SSI.getMFlow());
	}
	
}
