package Elements;
/**
 * LAMEE improves HX class to manage an entropy exchanger with air and a desiccant
 * solution, based in the last references of research.
 * 
 * @see HX
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class LAMEE extends HX{
	
	protected double UA_m;
	
	/**
	 * Creates an instance of LAMEE with latent UA of 0.0 W/kg, sensible
	 * UA of 0.0 W/K and fixed latent and sensible effectiveness of 1.0
	 */
	public LAMEE(){
		DSI.setFluid(new Air());
		SSI.setFluid(new Desiccant());
		UA = 0.0;
		UA_m = 0.0;
		EffL = 1.0;
		EffS = 1.0;
		SPL = 0.3;
		DPL = 0.02;
		fixed = true;
	}
	
	/**
	 * Adjust the latent UA
	 * 
	 * @param val Latent UA double value (W/kg)
	 */
	public void setUA_m(double val){
		fixed = false;
		UA_m = val;
		refresh();
	}
	
	/**
	 * Latent UA
	 * 
	 * @return Latent UA double value (W/kg)
	 */
	public double getUA_m(){
		return UA_m;
	}
	
	/**
	 * Obtain an approximation of E_t value defined by Zhang based on 
	 * specified Salt and temperature
	 * 
	 * @param d Desiccant instance to evaluate
	 * @param tem Temperature to evaluate (C)
	 * @return Et double value
	 * 
	 * @see Desiccant
	 */
	private double getEt(Fluid d, double tem){
		try{
		double result, w_f, w_o;
		d.setTem(tem + 0.001);
		w_f = d.getHum();
		d.setTem(tem - 0.001);
		w_o = d.getHum();
		result = (w_f - w_o) / 0.002;
		return result;
		} catch(OutOfRangeException ex){
			System.out.println(ex.getMessage());
			System.out.println("At: LAMEE.getEt()");
			return 0.0;
		}
	}
	
	/**
	 * Adjust sensible and latent effectiveness when are not fixed
	 */
	protected void setEff(){
		double R1 = (SSI.getTem() - DSI.getTem()) / (SSI.getHum() - DSI.getHum());
		double Et = getEt(SSI.getFluid(), SSI.getTem());
		double a[][] = new double[2][2];
		double b[][] = new double[2][3];
		double lambda[] = new double[2];
		double C[] = new double[2];
		double K[] = new double[2];
		double msen = DSI.getMFlow() * DSI.getCp()/ SSI.getMFlow() / SSI.getCp();
		double habs = (2501 + 1.86 * Math.max(DSI.getTem(), SSI.getTem()));
		double mabs = DSI.getMFlow() * habs * (SSI.getHum() - DSI.getHum())
						/ (SSI.getMFlow() * SSI.getCp() * (SSI.getTem() - DSI.getTem()));
		double NTUsen = UA / DSI.getMFlow() / DSI.getCp() / 1000.0;
		double NTUlat = DSI.getDensity() * UA_m / DSI.getMFlow();
		double Le_tot = NTUsen / NTUlat;
		a[0][0] = (msen - 1.0) * NTUsen;
		a[0][1] = mabs * NTUsen / Le_tot;
		a[1][0] = Et * R1 * msen * NTUsen;
		a[1][1] = (Et * R1 * mabs - 1) * NTUsen / Le_tot;
		lambda[0] = 0.5 * (a[0][0] + a[1][1] 
				+ Math.sqrt(Math.pow((a[0][0] + a[1][1]), 2.0) 
				- 4.0 * (a[0][0] * a[1][1] - a[0][1] * a[1][0])));
		lambda[1] = 0.5 * (a[0][0] + a[1][1] 
				- Math.sqrt(Math.pow((a[0][0] + a[1][1]), 2.0) 
				- 4.0 * (a[0][0] * a[1][1] - a[0][1] * a[1][0])));
		K[0] = a[1][0] / (a[1][1] - lambda[0]);
		K[1] = a[1][0] / (lambda[1] - a[1][1]);
		b[0][0] = K[0] + Et * R1;
		b[0][1] = Et * R1 - K[1];
		b[0][2] = 1 - Et * R1;
		b[1][0] = msen * Math.exp(lambda[0]) - mabs * K[0] * Math.exp(lambda[0]) - 1.0;
		b[1][1] = msen * Math.exp(lambda[1]) + mabs * K[1] * Math.exp(lambda[1]) - 1.0;
		b[1][2] = 1 - msen - mabs;
		C[0] = (b[0][2] * b[1][1] - b[0][1] * b[1][2]) 
				/ (b[0][1] * b[1][0] - b[0][0] * b[1][1]);
		C[1] = (b[0][0] * b[1][2] - b[0][2] * b[1][0])
				/ (b[0][1] * b[1][0] - b[0][0] * b[1][1]);
		EffL = 1 
			+ K[0] * C[0] * Math.exp(lambda[0]) - K[1] * C[1] * Math.exp(lambda[1]);
		EffS = 1 - C[0] * Math.exp(lambda[0]) - C[1] * Math.exp(lambda[1]);
	}
	
	/**
	 * Update all properties when inlet nodes are changed
	 */
	@Override
	protected void refresh(){
		if (! fixed){
			setEff();
		}
		DSO.setFluid(DSI.getFluid());
		SSO.setFluid(SSI.getFluid());
		DSO.setPressure(DSI.getPressure() - SPL);
		SSO.setPressure(SSI.getPressure() - DPL);
		double C_d = DSI.getMFlow() * DSI.getCp();
		double C_s = SSI.getMFlow() * SSI.getCp();
		double C_min = Math.min(C_d, C_s);
		Air air = (Air) DSI.getFluid();
		air.setRH(1.0);
		double Wmax = Math.min(air.getHum(), SSI.getHum());
		double dM , dH;
		DSO.setTem(DSI.getTem() 
				+ getEffS() * C_min / C_d * (SSI.getTem() - DSI.getTem()));
		DSO.setXCon(Air.getXConForW(DSI.getHum() 
				+ getEffL() * (Wmax - DSI.getHum())));
		dM = (DSI.getHum() - DSO.getHum()) / (1 + DSI.getHum()) * DSI.getMFlow();
		dH = DSO.getSpEnthalpy() - DSI.getSpEnthalpy();
		SSO.setTem(SSI.getTem()
				- (dH - SSI.getHabs() * dM) / SSI.getCp());
		DSO.setMFlow(DSI.getMFlow() - dM);
		SSO.setMFlow(SSI.getMFlow() + dM);
		SSO.setXCon(SSI.getXCon() * SSI.getMFlow() / SSO.getMFlow());
	}

	
}
