package Elements;

/**
 * LAMEE improves HX class to manage an entropy exchanger with air and a desiccant
 * solution, based in the last references of research.
 * 
 * @see HX
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class LAMEE2 extends HX{
	protected double msen;
	protected double mabs;
	protected double NTUsen;
	protected double Le_tot;
	protected boolean fixed;
	
	/**
	 * Creates an instance of LAMEE with latent UA of 0.0 W/kg, sensible
	 * UA of 0.0 W/K and fixed latent and sensible effectiveness of 1.0
	 */
	public LAMEE2(){
		msen = 1.0;
		mabs = 1.0;
		NTUsen = 1.0;
		Le_tot = 1.0;
		EffL = 1.0;
		EffS = 1.0;
	}
	
	/**
	 * Adjust the sensible effectiveness in a fixed value.
	 * 
	 * @param eff Sensible effectiveness double value.
	 */
	@Override
	public void setEffS(double eff){
		EffS = eff;
		fixed = true;
		refresh();
	}
	
	/**
	 * Adjust the latent effectiveness in a fixed value.
	 * 
	 * @param eff Latent effectiveness double value.
	 */
	@Override
	public void setEffL(double eff){
		EffL = eff;
		fixed = true;
		refresh();
	}
	
	
	public void setMabs(double val){
		fixed = false;
		mabs = val;
		refresh();
	}
	
	public void setMsen(double val){
		msen = val;
		fixed = false;
		refresh();
	}
	
	public double getMabs(){
		return mabs;
	}
	
	public double getMsen(){
		return msen;
	}
	
	public void setNTUsen(double val){
		NTUsen = val;
		fixed = false;
		refresh();
	}
	
	public void setLe_tot(double val){
		Le_tot = val;
		fixed = false;
		refresh();
	}
	
	public double getNTUsen(){
		return NTUsen;
	}
	
	public double getLe_tot(){
		return Le_tot;
	}
	
	/**
	 * Obtain an approximation of E_t value defined by Zhang based on 
	 * specified Salt and temperature
	 * 
	 * @param d Desiccant instance to evaluate
	 * @param tem Temperature to evaluate (C)
	 * @return Et double value
	 * 
	 * @see Desiccant
	 */
	private double getEt(Fluid d, double tem){
		try{
		double result, w_f, w_o;
		d.setTem(tem + 0.001);
		w_f = d.getHum();
		d.setTem(tem - 0.001);
		w_o = d.getHum();
		result = (w_f - w_o) / 0.002;
		return result;
		} catch (OutOfRangeException ex){
			System.out.println(ex.getMessage());
			return 0.0;
		}
	}
	
	/**
	 * Adjust sensible and latent effectiveness when are not fixed
	 */
	protected void setEff(){
		System.out.println("//////////////////////////////////////////////////");
		double R1 = (SSI.getTem() - DSI.getTem()) / (SSI.getHum() - DSI.getHum());
		//double R1 = (SSI.getTem() - DSI.getTem()) / (0.0055 - DSI.getHum());
		System.out.println("R1: " + R1);
		//double Et = getEt(SSI.getFluid(), SSI.getTem());
		double Et = 5.39E-4;
		System.out.println("Et: " + Et);
		double a[][] = new double[2][2];
		double b[][] = new double[2][3];
		double lambda[] = new double[2];
		double C[] = new double[2];
		double K[] = new double[2];
		a[0][0] = (msen - 1.0) * NTUsen;
		System.out.println("a11: " + a[0][0]);
		a[0][1] = mabs * NTUsen / Le_tot;
		System.out.println("a12: " + a[0][1]);
		a[1][0] = Et * R1 * msen * NTUsen;
		System.out.println("a21: " + a[1][0]);
		a[1][1] = (Et * R1 * mabs - 1) * NTUsen / Le_tot;
		System.out.println("a22: " + a[1][1]);
		System.out.println();
		lambda[0] = 0.5 * (a[0][0] + a[1][1] 
				+ Math.sqrt(Math.pow((a[0][0] + a[1][1]), 2.0) 
				- 4.0 * (a[0][0] * a[1][1] - a[0][1] * a[1][0])));
		System.out.println("lmd1: " + lambda[0]);
		lambda[1] = 0.5 * (a[0][0] + a[1][1] 
				- Math.sqrt(Math.pow((a[0][0] + a[1][1]), 2.0) 
				- 4.0 * (a[0][0] * a[1][1] - a[0][1] * a[1][0])));
		System.out.println("lmd2: " + lambda[1]);
		System.out.println();
		K[0] = a[1][0] / (a[1][1] - lambda[0]);
		System.out.println("K0: " + K[0]);
		K[1] = a[1][0] / (lambda[1] - a[1][1]);
		System.out.println("K1: " + K[1]);
		System.out.println();
		b[0][0] = K[0] + Et * R1;
		System.out.println("b11: " + b[0][0]);
		b[0][1] = Et * R1 - K[1];
		System.out.println("b12: " + b[0][1]);
		b[0][2] = 1 - Et * R1;
		System.out.println("b13: " + b[0][2]);
		b[1][0] = msen * Math.exp(lambda[0]) - mabs * K[0] * Math.exp(lambda[0]) - 1.0;
		System.out.println("b21: " + b[1][0]);
		b[1][1] = msen * Math.exp(lambda[1]) + mabs * K[1] * Math.exp(lambda[1]) - 1.0;
		System.out.println("b22: " + b[1][1]);
		b[1][2] = 1 - msen - mabs;
		System.out.println("b23: " + b[1][2]);
		System.out.println();
		C[0] = (b[0][2] * b[1][1] - b[0][1] * b[1][2]) 
				/ (b[0][1] * b[1][0] - b[0][0] * b[1][1]);
		System.out.println("C0: " + C[0]);
		C[1] = (b[0][0] * b[1][2] - b[0][2] * b[1][0])
				/ (b[0][1] * b[1][0] - b[0][0] * b[1][1]);
		System.out.println("C1: " + C[1]);
		System.out.println();
		EffL = 1 
			+ K[0] * C[0] * Math.exp(lambda[0]) - K[1] * C[1] * Math.exp(lambda[1]);
		EffS = 1 - C[0] * Math.exp(lambda[0]) - C[1] * Math.exp(lambda[1]);
	}
	
	/**
	 * Update all properties when inlet nodes are changed
	 */
	@Override
	protected void refresh(){
		if (! fixed){
			setEff();
		}
		DSO.setFluid(DSI.getFluid());
		SSO.setFluid(SSI.getFluid());
		DSO.setPressure(DSI.getPressure());
		SSO.setPressure(SSI.getPressure());
		double C_d = DSI.getMFlow() * DSI.getCp();
		double C_s = SSI.getMFlow() * SSI.getCp();
		double C_min = Math.min(C_d, C_s);
		double dM;
		DSO.setTem(DSI.getTem() 
				+ getEffS() * C_min / C_d * (SSI.getTem() - DSI.getTem()));
		SSO.setTem(SSI.getTem()
				- getEffS() * C_min / C_s * (SSI.getTem() - DSI.getTem()));
		DSO.setXCon(Air.getXConForW(DSI.getHum() 
			+ getEffL() * (SSI.getHum() - DSI.getHum())));
		dM = (DSI.getHum() - DSO.getHum()) / (1 + DSI.getHum()) * DSI.getMFlow();
		DSO.setMFlow(DSI.getMFlow() - dM);
		SSO.setMFlow(SSI.getMFlow() + dM);
		SSO.setXCon(SSI.getXCon()* SSI.getMFlow() / SSO.getMFlow());
	}
	
}
