package Elements;
/**
 * Node is the element of connections, it saves the type of fluid and the temperature,
 * pressure, concentration and flow mass.
 * 
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class Node {
	private Fluid a_fluid;
	private double a_P;
	private double a_Tem;
	private double a_XCon;
	private double a_MF;
	
	/**
	 * Creates a new instance of Node with a basid FMix fluid, an pressure of 
	 * 101.325 kPa, 20.0 C of temperature, 
	 * 0.0 of concentration, and 0.0 kg/s of mass flow.
	 * 
	 * @see FMix
	 */
	public Node(){
		a_fluid = new Fluid();
		a_P = 101.325;
		a_Tem = 20.0;
		a_XCon = 0.0;
		a_MF = 0.0;
	}
	
	/**
	 * Creates a new Node instance with a specified FMix fluid and fixed temperature
	 * (C), concentration (kg_salt/kg_sol), mass flow (kg/s), with a pressure of
	 * 101.325 kPa.
	 * 
	 * @param fluid FMix fluid that pass through the Node
	 * @param tem Temperature of fluid in Node (C)
	 * @param con Concentration of salt in the desiccant (kg_salt/kg_sol)
	 * @param mf Mass flow of fluid (kg/s)
	 * 
	 * @see FMix
	 */
	public Node(Fluid fluid, double tem, double con, double mf){
		a_P = 101.325;
		a_fluid = fluid;
		a_Tem = tem;
		a_XCon = con;
		a_MF = mf;
	}
	
	/**
	 * Adjust the FMix fluid to use
	 * 
	 * @param fluid Instance of FMix
	 * @see FMix
	 */
	public void setFluid(Fluid fluid){
		a_fluid = fluid;
	}
	
	/**
	 * FMix Fluid
	 * 
	 * @return FMix fluid
	 * 
	 * @see FMix
	 */
	public Fluid getFluid(){
		return a_fluid;
	}
	
	/**
	 * Adjust the temperature of fluid (C)
	 * 
	 * @param tem Temperature double value (C)
	 */
	public boolean setTem(double tem){
		double backup = a_Tem;
		a_Tem = tem;
		try{
		verifyFluid();
		return true;
		} catch(OutOfRangeException ex){
			a_Tem = backup;
			return false;
		}
	}
	
	/**
	 * Temperature of fluid (C)
	 * 
	 * @return Temperature double value (C)
	 */
	public double getTem(){
		return a_Tem;
	}
	
	/**
	 * Adjust the concentration of solute in the fluid solution (kg_solute/kg_solution)
	 * 
	 * @param con Concentration of solute double value (kg_solute/kg_solution)
	 */
	public boolean setXCon(double con){
		double backup = a_XCon;
		a_XCon = con;
		try{
			verifyFluid();
			return true;
		} catch(OutOfRangeException ex){
			a_XCon = backup;
			return false;
		}
	}
	
	/**
	 * Concentration of solute in the fluid solution (kg_solute/kg_solution)
	 * 
	 * @return Concentration of solute double value (kg_solute/kg_solution)
	 */
	public double getXCon(){
		return a_XCon;
	}
	
	/**
	 * Equilibrium humidity ration of fluid solution (kg_w/kg_da)
	 * 
	 * @return Equilibrium humidity ratio double value (kg_w/kg_da)
	 */
	public double getHum(){
		adjustFluid();
		return a_fluid.getHum();
	}
	
	/**
	 * Adjust the absolute pressure of fluid (kPa)
	 * 
	 * @param pre Absolute pressure double value (kPa)
	 */
	public boolean setPressure(double pre){
		double backup = a_P;
		a_P = pre;
		try {
		verifyFluid();
		return true;
		} catch(OutOfRangeException ex){
			a_P = backup;
			return false;
		}
	}
	
	/**
	 * Absolute pressure of fluid (kPa)
	 * 
	 * @return Absolute pressure double value (kPa)
	 */
	public double getPressure(){
		return a_P;
	}
	
	/**
	 * Adjust fluid mass flow (kg/s)
	 * 
	 * @param mf Fluid mass flow double value (kg/s)
	 */
	public boolean setMFlow(double mf){
		if (mf >= 0.0){
			a_MF = mf;
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Fluid mass flow (kg/s)
	 * 
	 * @return Fluid mass flow double value (kg/s)
	 */
	public double getMFlow(){
		return a_MF;
	}
	
	/**
	 * Fluid density (kg/m^3)
	 * 
	 * @return Fluid density double value (kg/m^3)
	 */
	public double getDensity(){
		adjustFluid();
		return a_fluid.getDensity();
	}
	
	/**
	 * Constant pressure specific heat (kJ/kg-K)
	 * 
	 * @return Constant pressure specific heat double value (kJ/kg-K)
	 */
	public double getCp(){
		adjustFluid();
		return a_fluid.getSpHeat();
	}
	
	/**
	 * Specific enthalpy of fluid (kJ/kg)
	 * 
	 * @return Specific enthalpy of fluid double value (kJ/kg)
	 */
	public double getSpEnthalpy(){
		adjustFluid();
		return a_fluid.getSpEnthalpy();
	}
	
	public double getHabs(){
		adjustFluid();
		return a_fluid.getHfg();
	}
	
	/**
	 * Dynamic viscosity of fluid (uPa-s)
	 * 
	 * @return Dynamic viscosity of fluid double value (uPa-s)
	 */
	public double getViscosity(){
		adjustFluid();
		return a_fluid.getViscosity();
	}
	
	/**
	 * Determines if the propertie changed is in the ranges allowed for calculations in the selected fluid
	 */
	public void verifyFluid() throws OutOfRangeException{
		try{
			a_fluid.setXCon(a_XCon);
		} catch(OutOfRangeException ex){
			// Hold the Exception until other two properties are settled
		}
		try {
			a_fluid.setPressure(a_P);
		} catch(OutOfRangeException ex){
			// Hold the Exception until other two porperties are settled
		}
		a_fluid.setTem(a_Tem);
	}
	
	/**
	 * Adjust the Fluid when either pressure, temperature or concetration are changed
	 */
	private void adjustFluid(){
		try{
		a_fluid.setXCon(a_XCon);
		a_fluid.setPressure(a_P);
		a_fluid.setTem(a_Tem);
		} catch(OutOfRangeException ex){
			// Filters the Exception caused when adjust the properties
		}
	}
}
