package Elements;
/**
 * Pipe simulates the pipe behavior, uses the physical properties of pipe and
 * thickness to evaluate heat and pressure loss.
 * 
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class Pipe {
	protected Node NI;
	protected Node NO;
	protected double a_Tamb;
	protected double a_Qloss;
	protected double a_Ploss;
	protected double p_Diameter;
	protected double p_Thickness;
	protected double i_Thickness;
	protected double a_kp;
	protected double a_ki;
	protected double a_hoo;
	protected double a_hf;
	protected double a_Length;
	protected double p_LFactor;
	protected double a_UFactor;
	protected Type a_Type;
	protected Insulation a_Ins;
	
	/**
	 * database of some common pipe type to use
	 */
	public enum Type{
		CUSTOM,
		PPR_PN16_20OD,
		PPR_PN16_25OD,
		PPR_PN16_32OD,
		PPR_PN16_40OD
	}
	
	/**
	 * database of some common insulation walls to use
	 */
	public enum Insulation{
		CUSTOM,
		AFX_127,
		AFX_254
	}
	
	/**
	 * 
	 * Creates an instance of the object Pipe with a PPR pipe OD 25 mm PN 16
	 * insulated with a 12.7 mm thickness of foam insulation.
	 */
	public Pipe(){
		NI = new Node();
		NO = new Node();
		a_Qloss = 0.0;
		a_Ploss = 0.0;
		a_hoo = 7.0;
		a_hf = 135.0;
		setType(Type.PPR_PN16_25OD);
		setInsulation(Insulation.AFX_127);
		a_Tamb = 40.0;
		a_Length = 10.0;
	}
	
	public Pipe(Type type, Insulation ins, double length){
		NI = new Node();
		NO = new Node();
		a_Qloss = 0.0;
		a_Ploss = 00;
		a_hoo = 7.0;
		a_hf = 135.0;
		setType(type);
		setInsulation(ins);
		a_Length = 10.0;
	}
	
	/**
	 * 
	 * Adjust the inlet node with the referenced node.
	 * 
	 * @param node from the inlet point of the pipe connection.
	 * 
	 * @see Node
	 */
	public void setNI(Node node) throws OutOfRangeException{
		NI.setFluid(node.getFluid());
		NI.setPressure(node.getPressure());
		NI.setTem(node.getTem());
		NI.setXCon(node.getXCon());
		NI.setMFlow(node.getMFlow());
		refresh();
	}
	
	/**
	 * 
	 * Get the inlet Node
	 * 
	 * @return An instance of inlet Node object in the pipe.
	 * 
	 * @see Node
	 */
	public Node getNI(){
		return NI;
	}
	
	/**
	 * 
	 * Get the outlet Node
	 * 
	 * @return An instance of outlet Node object in the pipe.
	 * 
	 * @see Node
	 */
	public Node getNO(){
		return NO;
	}
	
	public boolean setHf(double hf){
		if (hf < 0.0){
			return false;
		} else {
			a_hf = hf;
			refresh();
			return true;
		}
	}
	
	public double getHf(){
		return a_hf;
	}
	
	public boolean setHoo(double hoo){
		if (hoo < 0.0){
			return false;
		} else {
			a_hoo = hoo;
			refresh();
			return true;
		}
	}
	
	public double getHoo(){
		return a_hoo;
	}
	
	/**
	 * 
	 * calculates and save the heat loss of pipe
	 * 
	 */
	protected void setQloss(){
		double r_id, r_od, r_oo;
		r_od = p_Diameter / 2.0;
		r_id = r_od - p_Thickness;
		r_oo = r_od + i_Thickness;
		a_UFactor = 1.0 / (1 / a_hf + r_id / a_kp * Math.log(r_od / r_id)
					+ r_id / a_ki * Math.log(r_oo / r_od) + r_id / r_oo / a_hoo);
		double A_i = 2 * Math.PI * r_id * a_Length;
		a_Qloss = a_UFactor * A_i * (a_Tamb - NI.getTem());
	}
	
	/**
	 * 
	 * Pipe heat loss (W)
	 * 
	 * @return Pipe heat loss value (W)
	 */
	public double getQloss(){
		return a_Qloss;
	}
	
	/**
	 * 
	 * Calculates and store the pressure loss in Pipe
	 * 
	 */
	protected void setPloss(){
		double D = p_Diameter - 2.0 * p_Thickness;
		double V_m = 4 * NI.getMFlow() / NI.getDensity() / Math.PI / D / D;
		a_Ploss = p_LFactor * a_Length / D * NI.getDensity() * V_m * V_m / 2000.0;
	}
	
	/**
	 * 
	 * Pipe pressure loss (kPa)
	 * 
	 * @return Pipe pressure loss value (kPa)
	 */
	public double getPloss(){
		return a_Ploss;
	}
	
	/**
	 * 
	 * Adjust the ambient drybulb temperature where the pipe is located (C)
	 * 
	 * @param tem Ambient drybulb temperature (C)
	 */
	public void setTamb(double tem){
		a_Tamb = tem;
		refresh();
	}
	
	/**
	 * 
	 * Ambient drybulb temperature where the pipe is located
	 * 
	 * @return Outside drybulb temperature value (C)
	 * 
	 */
	public double getTamb(){
		return a_Tamb;
	}
	
	/**
	 * 
	 * Adjust the length of pipe
	 * 
	 * @param L Pipe length value(m)
	 */
	public void setLength(double L){
		a_Length = L;
		refresh();
	}
	
	/**
	 * 
	 * Pipe length (m)
	 * 
	 * @return Pipe length value (m)
	 */
	public double getLength(){
		return a_Length;
	}
	
	/**
	 * 
	 * Adjust the Pipe thermal conductivity (W/m-K)
	 * 
	 * @param k Pipe thermal conductivity value (W/m-K)
	 */
	public void setPpThCond(double k){
		a_kp = k;
		refresh();
	}
	
	/**
	 * 
	 * Pipe thermal conductivity (W/m-K)
	 * 
	 * @return Pipe thermal conductivity value (W/m-K)
	 */
	public double getPpThCond(){
		return a_kp;
	}
	
	/**
	 * 
	 * Adjust the pipe insulation thermal conductivy (W/m-K)
	 * 
	 * @param k Insulation thermal conductivity value (W/m-K) 
	 */
	public void setInThCond(double k){
		a_ki = k;
		refresh();
	}
	
	/**
	 * 
	 * Insulation thermal conductivity (W/m-K)
	 * 
	 * @return Insulation Thermal conductivity value (W/m-K)
	 */
	public double getInThCond(){
		return a_ki;
	}
	
	/**
	 * 
	 * Adjusts the outside pipe diameter (m)
	 * 
	 * @param D Pipe outside diameter value (m)
	 */
	public void setOD(double D){
		p_Diameter = D;
		refresh();
	}
	
	/**
	 * 
	 * Outside diameter dimension (m)
	 * 
	 * @return Pipe outside diameter value (m)
	 */
	public double getOD(){
		return p_Diameter;
	}
	
	/**
	 * 
	 * Adjust the pipe thickness (m)
	 * 
	 * @param t Pipe thickness double value (m)
	 */
	public void setPpThick(double t){
		p_Thickness = t;
		refresh();
	}
	
	/**
	 * 
	 * Pipe thickness (m)
	 * 
	 * @return Pipe thickness double value (m)
	 */
	public double getPpThick(){
		return p_Thickness;
	}
	
	/**
	 * 
	 * Adjust the insulation thickness (m)
	 * 
	 * @param t Insulation thickness double value (m)
	 */
	public void setInThick(double t){
		i_Thickness = t;
		refresh();
	}
	
	/**
	 * 
	 * Insulation thickness (m)
	 * 
	 * @return Insulation thickness double value (m)
	 */
	public double getInThick(){
		return i_Thickness;
	}
	
	/**
	 * Adjust the Pipe loss factor
	 * 
	 * @param factor Pipe loss factor coefficient DIN 53483
	 */
	public void setPpLFactor(double factor){
		p_LFactor = factor;
		refresh();
	}
	
	/**
	 * Obtain the Pipe loss factor coefficient
	 * 
	 * @return Pipe loss factor coefficient DIN 53483
	 */
	public double getPpLFactor(){
		return p_LFactor;
	}
	
	/**
	 * Select all pipe properties based on the internal database of posible pipes
	 * 
	 * @param type Type of pipe
	 */
	public void setType(Type type){
		a_Type = type;
		if (type == Type.PPR_PN16_40OD){
			setOD(0.040);
			setPpThick(0.0056);
			setPpThCond(0.24);
			setPpLFactor(0.02526);
		} else if (type == Type.PPR_PN16_32OD){
			setOD(0.032);
			setPpThick(0.0045);
			setPpThCond(0.24);
			setPpLFactor(0.02526);
		} else if (type == Type.PPR_PN16_25OD){
			setOD(0.025);
			setPpThick(0.0035);
			setPpThCond(0.24);
			setPpLFactor(0.02526);
		} else {
			setOD(0.020);
			setPpThick(0.0028);
			setPpThCond(0.24);
			setPpLFactor(0.02526);
		}
		refresh();
	}
	
	public Type getType(){
		return a_Type;
	}
	
	/**
	 * Adjust the Insulation characteristics based on an internal database
	 * 
	 * @param ins Insulation wall type
	 */
	public void setInsulation(Insulation ins){
		a_Ins = ins;
		if(ins == Insulation.AFX_254){
			setInThick(0.0254);
			setInThCond(0.036);
		} else {
			setInThick(0.0127);
			setInThCond(0.036);
		}
		refresh();
	}
	
	public Insulation getInsulation(){
		return a_Ins;
	}
	
	public void display(){
		System.out.println("CARACTERSTICAS DE LA TUBERA");
		System.out.println("Dimetro externo OD: " + String.valueOf(getOD() * 1000.0)+ " mm");
		System.out.println("Dimetro interno ID: " + String.valueOf((getOD() - 2.0 * getPpThick()) * 1000.0) + " mm");
		System.out.println("Longitud L: " + String.valueOf(getLength()) + " m");
		System.out.println("Espesor del aislamiento w: " + String.valueOf(getInThick() * 1000.0) + " mm");
		System.out.println("Conductividad de la tubera k_p: " + String.valueOf(getPpThCond()) + " W/m-K");
		System.out.println("Conductividad del aislamiento k_w: " + String.valueOf(getInThCond()) + " W/m-K");
		System.out.println("Flujo msico m: " + String.valueOf(NI.getMFlow()) + " kg/s");
		System.out.println("Gasto volumtrico Vv: " + String.valueOf(NI.getMFlow()/NI.getDensity()) + " m^3/s");
		System.out.println("Densidad del fluido rho: " + NI.getDensity() + " kg / m^3");
		System.out.println("Calor especfico del fluido Cp: " + NI.getCp() +" kJ/kg-K");
		System.out.println("Viscosidad dinmica del fluido u: " + NI.getViscosity() + " uPa-s");
		System.out.println("Factor U de T.C.: " + a_UFactor + " W/m^2-K");
		System.out.println("Prdidas de presin en la tubera dP: " + String.valueOf(a_Ploss) + " kPa");
		System.out.println("Prdidas de calor en la tubera dQ: " + String.valueOf(a_Qloss) + " W");
		System.out.println("Temperatura ambiente T_amb: " + getTamb() + " C");
		System.out.println("Temperatura de entrada T_i: " + NI.getTem() + " C");
		System.out.println("Temperatura de salida T_o: " + NO.getTem() + " C");
	}
	
	/**
	 * After any change, update the outside nodes characteristics
	 */
	public void refresh(){
		setQloss();
		setPloss();
		NO.setFluid(NI.getFluid());
		NO.setPressure(NI.getPressure() - a_Ploss);
		NO.setTem(NI.getTem() + a_Qloss / NI.getMFlow() / NI.getCp() / 1000.0);
		NO.setXCon(NI.getXCon());
		NO.setMFlow(NI.getMFlow());
	}
}
