package Elements;
/**
 * Pump simulates an ideal Pump with specified efficiency and a determined
 * working curve.
 * 
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class Pump {
	protected double a_eSH;
	protected double a_eMT;
	protected boolean a_fixed;
	protected Node a_NI;
	protected Node a_NO;
	protected double a_H;
	protected PumpPerformedCurve a_PPC;
	protected double[] a_Cx;
	
	/**
	 * 
	 */
	public Pump(){
		a_eSH = 0.9;
		a_eMT = 0.9;
		a_fixed = true;
		a_NI = new Node();
		a_NO = new Node();
		a_H = 300.0;
		a_PPC = new PumpPerformedCurve();
		refresh();
	}
	
	/**
	 * It adjusts shaft efficiency
	 * 
	 * @param eta shaft efficiency double value (0.0 - 1.0)
	 * @throws OutOfRangeException 
	 */
	public boolean setESH(double eta){
		if(eta < 0.0 || eta > 1.0){
			return false;
		}
		a_eSH = eta;
		refresh();
		return true;
	}
	
	/**
	 * It obtains shaft efficiency
	 * 
	 * @return shaft efficiency double value (0.0 - 1.0)
	 */
	public double getESH(){
		return a_eSH;
	}
	
	/**
	 * It adjusts motor efficiency
	 * 
	 * @param eta motor efficiency (0.0 - 1.0)
	 */
	public boolean setEMT(double eta){
		if (eta < 0.0 && eta > 1.0){
			return false;
		}
		a_eMT = eta;
		refresh();
		return true;
	}
	
	/**
	 * It obtains motor efficiency
	 * 
	 * @return motor efficiency (0.0 - 1.0)
	 */
	public double getEMT(){
		return a_eMT;
	}
	
	/**
	 * It adjusts Inlet Side Node
	 * 
	 * @param NI
	 */
	public void setNI(Node NI){
		a_NI.setFluid(NI.getFluid());
		a_NI.setMFlow(NI.getMFlow());
		a_NI.setPressure(NI.getPressure());
		a_NI.setTem(NI.getTem());
		a_NI.setXCon(NI.getXCon());
		refresh();
	}
	
	/**
	 * It obtains Inlet Side Node
	 * 
	 * @return Inlet Side Node
	 * @see Node
	 */
	public Node getNI(){
		return a_NI;
	}
	
	/**
	 * It obtains Outlet Side Node
	 * 
	 * @return Outlet Side Node
	 * @see Node
	 */
	public Node getNO(){
		return a_NO;
	}
	
	/**
	 * It adjust the volume flow based on a specified Pump's head and the
	 * Pump's performance curve
	 * 
	 * @throws OutOfRangeException
	 */
	protected boolean setQ(){
		double q = a_NI.getMFlow() / a_NI.getDensity();
		if (q < a_PPC.getQmin() || q > a_PPC.getQmax()){
			return false;
		}
		try{
		a_NI.setMFlow(a_Cx[1] * a_PPC.getQ(a_H / a_Cx[2], a_NI.getDensity()) * a_NI.getDensity());
		return true;
		} catch(OutOfRangeException ex){
			return false;
		}
	}
	
	/**
	 * It obtains Pump's volume flow
	 * 
	 * @return volume flow double value (L/s)
	 */
	public double getQ(){
		return a_NI.getMFlow() / a_NI.getDensity() * 1000.0;
	}
	
	/**
	 * It adjusts Pump's head
	 * 
	 * @param H double value of Pump's head (kPa)
	 */
	public void setH(double H){
		a_H = H;
		refresh();
	}
	
	/**
	 * It obtains Pump's head
	 * 
	 * @return double value of Pump's head (kPa)
	 */
	public double getH(){
		return a_H;
	}
	
	/**
	 * It adjust the way Pump calculates the outlet Node properties.
	 * 
	 * @param fixed
	 */
	public void setPPC(PumpPerformedCurve PPC){
		a_PPC = PPC;
		a_fixed = false;
		refresh();
	}
	
	/**
	 * It returns if Pump is calculating the outlet properties with calculated values 
	 * (false) or with fixed values (true)
	 * 
	 * @return boolean value with the status
	 */
	public PumpPerformedCurve getPPC(){
		return a_PPC;
	}
	
	/**
	 * It obtains the real Pump's power consumption
	 * 
	 * @return Pump's power consumption double value (W)
	 * 
	 */
	public double getPMPPower(){
		return getQ() * a_H * 1000.0 / a_eSH / a_eMT / a_Cx[0];
	}
	

	
	/**
	 * It updates the volume flow based on the Pump's performance curve
	 */
	protected void refresh(){
		if(a_fixed){
			a_Cx = new double[3];
			a_Cx[0] = 1.0;
			a_Cx[1] = 1.0;
			a_Cx[2] = 1.0;
		} else{
			a_Cx = a_PPC.getViscFactors(a_NI.getViscosity(), a_NI.getDensity());
			if (setQ() == false){
				a_fixed = true;
			}
		}
		a_NO.setFluid(a_NI.getFluid());
		a_NO.setPressure(a_NI.getPressure() + a_H);
		a_NO.setMFlow(a_NI.getMFlow());
		a_NO.setXCon(a_NI.getXCon());
		a_NO.setTem(a_NI.getTem());
	}
}
