package Elements;

public class PumpPerformedCurve {
	protected double[] a_C;
	protected double[] a_Q;
	protected double a_QBEP;
	protected double a_N;
	protected double a_D;
	
	public PumpPerformedCurve(){
		a_C = new double[5];
		a_C[0] = 0.6;
		a_C[1] = 4.0;
		a_C[2] = -10.0;
		a_C[3] = 0.0;
		a_C[4] = 0.0;
		a_Q = new double[2];
		a_Q[0] = 0.0;
		a_Q[1] = 0.5;
		a_QBEP = 0.3;
		a_N = 1.0;
		a_D = 1.0;
	}
	
	public boolean setN(double N){
		if (N > 0.0){
			a_N = N;
			return true;
		} else{
			return false;
		}
	}
	
	public double getN(){
		return a_N;
	}
	
	public boolean setD(double D){
		if (D > 0.0){
			a_D = D;
			return true;
		} else{
			return false;
		}
	}
	
	public double getD(){
		return a_D;
	}
	
	public boolean setQmin(double q){
		if (q < a_Q[1] && q < a_QBEP){
			a_Q[0] = q;
			return true;
		} else {
			return false;
		}
	}
	
	public double getQmin(){
		return a_Q[0];
	}
	
	public boolean setQmax(double q){
		if (q > a_Q[0] && q > a_QBEP){
			a_Q[1] = q;
			return true;
		} else {
			return false;
		}
	}
	
	public double getQmax(){
		return a_Q[1];
	}
	
	/**
	 * It adjusts the five constants of the Pump's curve polynomial equation
	 * 
	 * @param C double array with constants of the polynomial equation
	 */
	public boolean setC(double[] C){
		if (verify(C)){
			a_C = C;
			return true;
		} else {
			return false;
		}
	}
	
	public boolean setC(int i, double Ci){
		double[] C = a_C;
		C[i] = Ci;
		if (verify(C)){
			a_C = C;
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * It obtains the five constants of the Pump's curve polynomial equation
	 * 
	 * @return double array with constants of the polynomial equation
	 */
	public double[] getC(){
		return a_C;
	}
	
	/**
	 * It adjusts the Best Efficiency Point Volume Flow
	 * 
	 * @param Q Best Efficiency Point Volume Flow (L/s)
	 */
	public boolean setQBEP(double Q){
		if (Q > a_Q[0] && Q < a_Q[1]){
			a_QBEP = Q;
			return true;
		} else{
			return false;
		}
	}
	
	/**
	 * It obtains the Best Efficiency Point Volume Flow
	 * 
	 * @return Best Efficiency Point Volume Flow (L/s)
	 */
	public double getQBEP(){
		return a_QBEP;
	}
	
	/**
	 * It obtains the Pump's performance curve scope in a specified point
	 * 
	 * @param x Point to evaluate
	 * @return Pump's performance curve scope value at the specified point
	 */
	protected double getScope(double x){
		return a_C[1] + 2 * a_C[2] * x + 3 * a_C[3] * x * x + 4.0 * a_C[4] * x * x * x;
	}
	
	protected double getScope(double x, double[] C){
		double sum = 0.0;
		for(int i = 1; i < C.length; i++){
			sum += (double) i * C[i] * Math.pow(x, (double) (i - 1));
		}
		return sum;
	}
	
	public double getH(double q, double den){
		double phi = q / a_N / Math.pow(a_D, 3.0);
		double psi = getPsi(phi, a_C);
		return psi * a_N * a_N * a_D * a_D * den;
	}
	
	public double getQ(double h, double den) throws OutOfRangeException{
		double phiHmax = getQatHMAX() / a_N / Math.pow(a_D, 3.0);
		double phiMax = a_Q[1] / a_N / Math.pow(a_D, 3.0);
		double psimax = getPsi(phiHmax, a_C);
		double psimin = getPsi(phiMax, a_C);
		double psi = h / den / a_N / a_N / a_D / a_D;
		if (psi < psimin || psi > psimax){
			throw new OutOfRangeException();
		}
		double step = (phiMax - phiHmax) / 1000.0;
		double phi = phiHmax;
		while(getPsi(phi, a_C) - psi >= 0.0 && phi < phiMax){
			phi += step;
		}
		return phi * a_N * Math.pow(a_D,  3.0);
	}
	
	public double getQatHMAX(){
		double step = (a_Q[1] - a_Q[0]) / 100.0;
		double Qmax = 0.0;
		double phi = 0.0;
		double psiMX = 0.0;
		for (double i = a_Q[0]; i < a_Q[1]; i += step){
			phi = i / a_N / Math.pow(a_D, 3.0);
			if (getPsi(phi, a_C) > psiMX){
				Qmax = i;
				psiMX = getPsi(phi, a_C);
			}
		}
		return Qmax;
	}
	
	protected double getPsi(double x, double[] C){
		double sum = 0.0;
		for (int i = 0; i < C.length; i++){
			sum += a_C[i] * Math.pow(x, (double) i);
		}
		return sum;
	}
	
	/**
	 * It adjust the viscous correction factors based on HI Standards
	 */
	public double[] getViscFactors(double vis, double den){
		double sum, LMD, HBEP, phi;
		double[] Cx = new double[3];
		phi = a_QBEP / a_N / Math.pow(a_D, 3.0);
		HBEP = a_C[0] + phi * a_C[1] + phi * phi * a_C[2] + Math.pow(phi, 3.0) * a_C[3]
				+ Math.pow(phi, 4.0) * a_C[4];
		HBEP *= a_N * a_N * a_D * a_D * den;
		LMD = 1.95 * Math.sqrt(vis / den) / Math.sqrt( 
				0.04739 * Math.pow(HBEP / 0.3048, 0.25746) 
				* Math.sqrt(a_QBEP * 60.0 / 3.78541));
		double[][] G = {{1.0522, -3.512E-2, -9.0394E-4, 2.2218E-4, -1.1986E-6, 1.9895E-7},
				{0.9873, 9.019E-3, -1.6233E-3, 7.7233E-5, -2.0528E-6, 2.1009E-8},
				{1.0103, -4.6061E-3, 2.4091E-4, -1.6912E-5, 3.2459E-7, -1.6611E-9},
				{1.0167, -8.3641E-3, 5.1288E-4, -2.9941E-5, 6.1644E-7, -4.0487E-9},
				{1.0045, -2.664E-3, -6.8292E-4, 4.9706E-5, -1.6522E-6, 1.9172E-8},
				{1.0175, -7.8654E-3, -5.6018E-4, 5.4967E-5, -1.9035E-6, 2.1615E-8}};
		double[] K = new double[6];
		for(int i = 0; i < K.length; i++){
			sum = 0.0;
			for(int j = 0; j < 6; j++){
				sum += G[i][j] * Math.pow(LMD, (double) j);
			}
			K[i] = sum;
		}
		Cx[0] = K[0];
		Cx[1] = K[1];
		if (K[1] < 0.6){
			Cx[2] = K[2];
		} else if (K[1] < 0.8){
			Cx[2] = K[3];
		} else if (K[1] < 1.0){
			Cx[2] = K[4];
		} else{
			Cx[2] = K[5];
		}
		return Cx;
	}
	
	private boolean verify(double[] C){
		double step = (a_Q[1] - a_Q[0]) / 100.0;
		double phi[] = new double[2];
		double n = 0.0;
		int flag = 0;
		while(flag < 2 && n * step < a_Q[1]){
			phi[0] = (a_Q[0] + n * step) / a_N / Math.pow(a_D, 3.0);
			phi[1] = (a_Q[0] + (n + 1) * step) / a_N / Math.pow(a_D, 3.0);
			if(getPsi(phi[0], C) < 0.0){
				return false;
			}
			System.out.println(phi[0] + "\t" + getScope(phi[1], C));
			if (getScope(phi[1], C) == 0.0){
				flag++;
			System.out.println(phi[0] + "\t" + getScope(phi[0], C) / getScope(phi[1], C));
			} else if (getScope(phi[0], C) / getScope(phi[1], C) < 0){
				flag++;
			}
			n += 1.0;
		}
		System.out.println(flag);
		if (flag >= 2) {
			return false;
		} else if (flag == 1){
			System.out.println(getScope(a_Q[0] / a_N / Math.pow(a_D, 3.0), C));
			if (getScope(a_Q[0] / a_N / Math.pow(a_D, 3.0), C) <= 0.0){
				return false;
			} else {
				return true;
			}
		} else { // (flag == 0)
			if (getScope(a_Q[0] / a_N / Math.pow(a_D, 3.0), C) > 0.0){
				return false;
			} else {
				return true;
			}
		}
	}
	
	
}
