package Elements;
/**
 * Water extends the FMix class to implement the equations specified in IAPWS
 * to calculate physical properties of pure Water.
 * 
 * @see FMix
 * @author	Eng. Ephraim Alonso Puerto
 * @version	%I%, %G%
 * @since 1.0
 */
public class Water extends Fluid{
	private static double Tc = 647.14; // 647.14
	private static double pc = 22.064; // MPa
	private static double rhoc = 322.0; // kg/m^3
	
	public Water(){
		a_name = "Water";
		a_Tem = 20.0;
		a_Pressure = 101.325;
		a_XCon = 0.0;
		a_Hum = 0.0;
	}
	
	/**
	 * It calculates and adjusts the specific heat of Mix (kJ/kg-K)
	 */
	@Override
	protected void setSpHeat(){
		double tau;
		double[][] b = {{830.54602, -1247.52013, -68.60350, 491.27650, -1.80692, -137.51511},
						{88.7891, -120.1958, -16.9264, 52.4654, 0.10826, 0.46988}};

		tau = (getTem() + 273.15) / 228.0 - 1.0;
		if (getTem() <= 0.0){
			a_SpHeat = b[0][0] + b[0][1] * Math.pow(tau, 0.02) + b[0][2] * Math.pow(tau,  0.04)
						+ b[0][3] * Math.pow(tau, 0.06) + b[0][4] * Math.pow(tau, 1.8)
						+ b[0][5] * Math.pow(tau, 8.0);
		} else{
			a_SpHeat = b[1][0] + b[1][1] * Math.pow(tau, 0.02) + b[1][2] * Math.pow(tau,  0.04)
					+ b[1][3] * Math.pow(tau, 0.06) + b[1][4] * Math.pow(tau, 1.8)
					+ b[1][5] * Math.pow(tau, 8.0);
		}
	}
	
	/**
	 * It calculates and adjusts the saturated pressure of Mix (kPa)
	 */
	@Override
	protected void setPs(){
		double lnX, tau;
		double[] a = {-7.85823, 1.83991, -11.7811, 22.6705, -15.9393, 1.77516};
		tau = 1.0 - (getTem() + 273.15) / Tc;
		lnX = Tc / (getTem() + 273.15) * (a[0] * tau + a[1] * Math.pow(tau, 1.5)
				+ a[2] * Math.pow(tau, 3.0) + a[3] * Math.pow(tau, 3.5) 
				+ a[4] * Math.pow(tau, 4.0) + a[5] * Math.pow(tau,  7.5));
		a_Ps = pc * Math.exp(lnX) * 1000.0;
	}
	
	@Override
	protected void setHfg(){
		a_Hfg = 2500.8 - 2.36 * getTem() + 0.0016 * getTem() * getTem() - 0.00006 * Math.pow(getTem(), 3.0);
	}
	
	/**
	 * Calculate the thermal conductivity of water (W / m - K)
	 * under the temperature evaluated.
	 * 
	 */
	protected void setThConductivity(){
		double th, ph;
		int i,j;
		th = (getTem() + 273.15) / Tc;
		ph = getDensity() / rhoc;
		double[] A = {2.443221E-3, 1.323095E-2, 6.770357E-3, -3.454586E-3, 4.096266E-4, 0.0};
		double[][] L = {{1.60397357, -0.646013523, 0.111443906, 0.102997357, -0.0504123634, 0.00609859258},
						{2.33771842, -2.78843778, 1.53616167, -0.463045512, 0.0832827019, -0.00719201245},
						{2.19650529, -4.54580785, 3.55777244, -1.40944978, 0.275418278, -0.0205938816},
						{-1.21051378, 1.60812989, -0.621178141, 0.0716373224, 0.0, 0.0},
						{-2.720337, 4.57586331, -3.18369245, 1.1168348, -0.19268305, 0.012913842}};
		double dTh = Math.abs(th - 1.0) + 0.00308976;
		double[] LMD = new double[2];
		LMD[0] = 10.0932 / Math.pow(dTh, 3.0 / 5.0);
		LMD[1] = 2.0 +  0.0822994 / Math.pow(dTh, 3.0 / 5.0);
		double[] lmdh = new double[2];
		lmdh[0] = 0.0;
		for(i = 0; i <= 4; i++){
			lmdh[0] += A[i] / Math.pow(th, (double) i);
		}
		lmdh[0] = Math.pow(th, 0.5) / lmdh[0];
		double ref;
		lmdh[1] = 0.0;
		for(i = 0; i <= 4; i++){
			ref = 0.0;
			for(j = 0; j <= 5; j++){
				ref += L[i][j] * Math.pow(ph - 1.0, (double) j);
			}
			lmdh[1] += Math.pow(1.0 / th - 1.0, (double) i) * ref;
		}
		lmdh[1] = Math.exp(ph * lmdh[1]);
		a_kConduct = lmdh[0] * lmdh[1] / 1000.0;
	}
	
	/**
	 * It uses the temperature to calculate the density of water (kg/m^3)
	 * 
	 */
	protected void setDensity(){
		double[] B = {1.9937718430, 1.0985211604, -0.5094492996, -1.7619124270,
						-44.9005480267, -723692.2618632};
		double tau = 1.0 - (getTem() + 273.15) / Tc;
		a_Density = rhoc * (1 + B[0] * Math.pow(tau, 1.0 / 3.0)
						+ B[1] * Math.pow(tau, 2.0 / 3.0) 
						+ B[2] * Math.pow(tau, 5.0 / 3.0)
						+ B[3] * Math.pow(tau, 16.0 / 3.0)
						+ B[4] * Math.pow(tau, 43.0 / 3.0)
						+ B[5] * Math.pow(tau, 110.0 / 3.0));
	}
	
	/**
	 * Calculates viscosity using IAPWS equations
	 */
	@Override
	protected void setViscosity(){
		double th = (getTem() + 273.15) / Tc;
		double ph = getDensity() / rhoc;
		double[] u = new double[2];
		double[][] H = {{5.20094E-1, 2.22531E-1, -2.81378E-1, 1.61913E-1, -3.25372E-2, 0.0, 0.0},
						{8.50895E-2, 9.99115E-1, -9.06851E-1, 2.57399E-1, 0.0, 0.0, 0.0},
						{-1.08374, 1.88797, -7.72479E-1, 0.0, 0.0, 0.0, 0.0},
						{-2.89555E-1, 1.26613, -4.89837E-1, 0.0, 6.98452E-2, 0.0, -4.35673E-3},
						{0.0, 0.0, -2.57040E-1, 0.0, 0.0, 8.72102E-3, 0.0},
						{0.0, 1.20573E-1, 0.0, 0.0, 0.0, 0.0, -5.93264E-4}};
		u[0] = 100 * Math.sqrt(th) / (1.67752 + 2.20462 / th 
						+ 0.6366564 / th / th - 0.241605 / th / th / th);
		double value;
		u[1] = 0.0;
		for (int i = 0; i <= 5; i++){
			value = 0.0;
			for(int j = 0; j <= 6; j++){
				value += H[i][j] * Math.pow(ph - 1, (double) j);
			}
			u[1] += value * Math.pow(1.0 / th - 1.0, (double) i);
		}
		u[1] = Math.exp(ph * u[1]);
		a_Viscosity = u[0] * u[1];	
	}
	
	/**
	 * It Calculates the adimensional temperature parameter
	 * 
	 * @param temp Actual temperature double value (C)
	 * 
	 * @return Adimensional temperature
	 */
	public static double getTH(double temp){
		return (temp + 273.15) / Tc;
	}
	
}
