package SIM;
/**
 * SIM is the primitival object that saves all methods and LDDS behavior
 * 
 * @author Eng. Ephraim A. Puerto
 * @version %I%.%G%
 * @since 1.0
 */


import Elements.*;

public class Sim {
	HX A;
	HX K;
	HX G;
	LAMEE D;
	LAMEE R;
	Pump P;
	Pipe KD;
	Pipe DP;
	Pipe PA;
	Pipe AG;
	Pipe GR;
	Pipe RA;
	Pipe AK;
	
	/**
	 * Default constructor, all objects are created with its default conditions
	 */
	public Sim(){
		A = new HX();
		G = new HX();
		K = new HX();
		D = new LAMEE();
		R = new LAMEE();
		P = new Pump();
		KD = new Pipe();
		DP = new Pipe();
		PA = new Pipe();
		AG = new Pipe();
		GR = new Pipe();
		RA = new Pipe();
		AK = new Pipe();
	}
	
	/**
	 * It adjusts the system pipes ambient temperature.
	 * 
	 * @param tem Ambient temperature double value (C)
	 */
	private void AdjustTamb(double tem){
		KD.setTamb(tem);
		DP.setTamb(tem);
		PA.setTamb(tem);
		AG.setTamb(tem);
		GR.setTamb(tem);
		RA.setTamb(tem);
		AK.setTamb(tem);
	}
	
	/**
	 * It adjusts the inlet conditions of outside air in LAMEE D and the ambient
	 * temperature for pipes of the system.
	 * 
	 * @see Node
	 * @param node Object Node with all the inlet conditions of outside air.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_tDi(Node node) throws OutOfRangeException{
		D.setDI(node);
		R.getDI().setPressure(D.getDI().getPressure());
		K.getDI().setPressure(D.getDI().getPressure());
		K.getSI().setPressure(D.getDI().getPressure());
		G.getSI().setPressure(D.getDI().getPressure());
		AdjustTamb(node.getTem());
	}
	
	/**
	 * Return the inlet conditions of outside air in LAMEE D.
	 * 
	 * @see Node
	 * @return Node object with all the inlet conditions of outside air
	 */
	public Node getN_tDi(){
		return D.getDI();
	}
	
	/**
	 * Return the outlet conditions of treated air at dehumidifier tower
	 * @see Node
	 * @return Node object with all the outlet conditions of treated air at
	 * dehumidifier tower.
	 */
	public Node getN_tDo(){
		return D.getDO();
	}
	
	/**
	 * It adjusts the liquid desiccant inlet conditions in dehumidifier tower.
	 * 
	 * @see Node
	 * @param node Node instance with liquid desiccant conditions from inlet
	 * at dehumidfier tower D.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_cDi(Node node) throws OutOfRangeException{
		D.setSI(node);
	}
	
	/**
	 * 
	 * Returns the liquid desiccant inlet conditions in dehumidifier tower.
	 * 
	 * @see Node
	 * @return a Node instance with all inlet properties of liquied desiccant
	 * at dehumidifier tower.
	 */
	public Node getN_cDi(){
		return D.getSI();
	}
	
	/**
	 * It adjusts the inlet liquid desiccant inlet conditions in pipe between
	 * desiccant tower and heat exchanger A.
	 * @param node Node instance with all the inlet properties in Pipe DP.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_hDo(Node node) throws OutOfRangeException{
		DP.setNI(node);
	}
	
	/**
	 * Inlet liquid desiccant conditions at pipe between desiccant tower and heat
	 * exchanger A.
	 * 
	 * @see Node
	 * @return a Node instance with all inlet properties in Pipe DA.
	 */
	public Node getN_hDo(){
		return DP.getNI();
	}
	
	/**
	 * Inlet liquid desiccant conditions at Pump inlet
	 * 
	 * @param a Node instance with all Pump inlet properties
	 */
	public void setN_hPi(Node node){
		P.setNI(node);
	}
	
	/**
	 * Inlet liquid desiccant conditions at Pump inlet
	 * 
	 * @return a Node instance with all Pump inlet properties
	 */
	public Node getN_hPi(){
		return P.getNI();
	}
	
	/**
	 * Inlet liquid desiccant conditions at pipe between Pump P and HX A
	 * 
	 * @param node A Node instance with all properties at Pipe PA
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range
	 */
	public void setN_hPo(Node node) throws OutOfRangeException{
		PA.setNI(node);
	}
	
	/**
	 * Inlet liquid desiccant conditions at pipe between Pump P and HX A
	 * 
	 * @return a Node instance with all inlet properties at Pipe PA
	 */
	public Node getN_hPo(){
		return PA.getNI();
	}
	
	/**
	 * It adjusts the liquid desiccant inlet conditions at the regenerator tower.
	 * 
	 * @see Node
	 * @param node Node instance with liquid desiccant inlet conditions at LAMEE R.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range
	 */
	public void setN_hRi(Node node) throws OutOfRangeException{
		R.setSI(node);
	}
	
	/**
	 * Liquid desiccant inlet conditions at the regenerator tower.
	 * 
	 * @see Node
	 * @return Node instance with liquid desiccant inlet conditions at LAMEE R.
	 */
	public Node getN_hRi(){
		return R.getSI();
	}
	
	/**
	 * It adjusts the liquid desiccant inlet conditions at pipe between the renerator
	 * tower and the heat exchanger A.
	 * 
	 * @see Node
	 * @param node Node instance with liquid desiccant inlet conditions at Pipe RA.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_cRo(Node node) throws OutOfRangeException{
		RA.setNI(node);
	}
	
	/**
	 * Liquid desiccant inlet conditions at pipe between the renerator tower and
	 * the heat exchanger A.
	 * 
	 * @see Node
	 * @return Node instance with liquid desiccant inlet conditions at Pipe RA.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public Node getN_cRo(){
		return RA.getNI();
	}
	
	/**
	 * It adjusts the return air inlet conditions at regenerator tower
	 * 
	 * @see Node
	 * @param node Node instance with return air conditions from inlet at LAMEE R.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_xRi(Node node) throws OutOfRangeException{
		R.setDI(node);
	}
	
	/**
	 * Air inlet conditions at regenerator tower
	 * 
	 * @see Node
	 * @return Node instance with return air conditions from inlet at LAMEE R.
	 */
	public Node getN_xRi(){
		return R.getDI();
	}
	
	/**
	 * Exhaust air conditions from outlet at regenerator tower.
	 * 
	 * @see Node
	 * @return Node instance with exhaust air conditions from outlet at LAMEE R.
	 */
	public Node getN_xRo(){
		return R.getDO();
	}
	
	/**
	 * It adjusts the liquid desiccant conditions from inlet at heat exchanger G.
	 * 
	 * @see Node
	 * @param node Node instance with liquid desiccant conditions from inlet at HX G.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_hGi(Node node) throws OutOfRangeException{
		G.setDI(node);
	}
	
	/**
	 * Liquid desiccant conditions from inlet at heat exchanger G.
	 * 
	 * @see Node
	 * @return Node instance with liquid desiccant conditions from inlet at HX G.
	 */
	public Node getN_hGi(){
		return G.getDI();
	}
	
	/**
	 * It adjusts the liquid desiccant conditions from inlet at pipe between heat
	 * exchanger G and regenerator tower R.
	 * 
	 * @see Node
	 * @param node Node instance with liquid desiccant conditions from inlet at Pipe GR.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_hGo(Node node) throws OutOfRangeException{
		GR.setNI(node);
	}
	
	/**
	 * Liquid desiccant conditions from inlet at pipe between heat exchanger G and
	 * regenerator tower R.
	 * 
	 * @see Node
	 * @return Node instance with liquid desiccant conditions from inlet at Pipe GR.
	 */
	public Node getN_hGo(){
		return GR.getNI();
	}
	
	/**
	 * It adjusts the water conditions from inlet at heat exchanger G.
	 * 
	 * @see Node
	 * @param node Node instance with water conditions from inlet at HX G.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_sGi(Node node) throws OutOfRangeException{
		G.setSI(node);
	}
	
	/**
	 * Water conditions from inlet at heat exchanger G.
	 * 
	 * @see Node
	 * @return Node instance with water conditions from inlet at HX G.
	 */
	public Node getN_sGi(){
		return G.getSI();
	}
	
	/**
	 * Water conditions from outlet at heat exchanger G.
	 * 
	 * @see Node
	 * @return
	 */
	public Node getN_sGo(){
		return G.getSO();
	}
	
	/**
	 * It adjusts the liquid desiccant conditions from inlet at heat exchanger K.
	 * 
	 * @see Node
	 * @param node Node instance with liquid desiccant conditions from inlet at HX K.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_cKi(Node node) throws OutOfRangeException{
		K.setSI(node);
	}
	
	/**
	 * Liquid desiccant conditions from inlet at heat exchanger K.
	 * 
	 * @see Node
	 * @return Node instance with liquid desiccant conditions from inlet at HX K.
	 */
	public Node getN_cKi(){
		return K.getSI();
	}
	
	/**
	 * It adjusts the liquid desiccant conditions from inlet at pipe between heat
	 * exchanger K and dehumidifier tower D.
	 * 
	 * @see Node
	 * @param node Node instance with liquid desiccant conditions from inlet at Pipe KD.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_cKo(Node node) throws OutOfRangeException{
		KD.setNI(node);
	}
	
	/**
	 * Liquid desiccant conditions from inlet at pipe between heat exchanger K and
	 * dehumidifier tower D.
	 * 
	 * @see Node
	 * @return Node instance with liquid desiccant conditions from inlet at Pipe KD.
	 */
	public Node getN_cKo(){
		return KD.getNI();
	}
	
	/**
	 * It adjusts the water conditions from inlet at heat exchanger K.
	 * 
	 * @see Node
	 * @param node Node instance with water conditions from inlet at HX K.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_wKi(Node node) throws OutOfRangeException{
		K.setDI(node);
	}
	
	/**
	 * Water conditions from inlet at heat exchanger K.
	 * 
	 * @see Node
	 * @return Node instance with water conditions from inlet at HX K.
	 */
	public Node getN_wKi(){
		return K.getDI();
	}
	
	/**
	 * Water conditions from outlet at heat exchanger K.
	 * @see Node
	 * @return Node instance with water conditions from outlet at HX K.
	 */
	public Node getN_wKo(){
		return K.getDO();
	}
	
	/**
	 * It adjusts the coolest liquid desiccant conditions from inlet at heat exchanger A.
	 * 
	 * @see Node
	 * @param node Node instance with coolest liquid desiccant conditions from inlet
	 * at HX A.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_cAi(Node node) throws OutOfRangeException{
		A.setDI(node);
	}
	
	/**
	 * Coolest liquid desiccant conditions from inlet at heat exchanger A.
	 * 
	 * @see Node
	 * @return Node instance with coolest liquid desiccant conditions from inlet
	 * at HX A.
	 */
	public Node getN_cAi(){
		return A.getDI();
	}
	
	/**
	 * It adjusts the liquid desiccant conditions from inlet at pipe between heat
	 * exchanger A and heat exchanger K.
	 * 
	 * @see Node
	 * @param node Node instance with coolest liquid desiccant conditions from inlet
	 * at HX A.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_cAo(Node node) throws OutOfRangeException{
		AK.setNI(node);
	}
	
	/**
	 * Liquid desiccant conditions from inlet at pipe between heat exchanger A and heat
	 * exchanger K.
	 * 
	 * @see Node
	 * @return Node instance with liquid desiccant conditions from inlet at PIpe AK.
	 */
	public Node getN_cAo(){
		return AK.getNI();
	}
	
	/**
	 * It adjusts the hottest liquid desiccant conditions from inlet at heat
	 * exchanger A.
	 * 
	 * @see Node
	 * @param node Node instance with hottest liquid desiccant conditions from HX A.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_hAi(Node node) throws OutOfRangeException{
		A.setSI(node);
	}
	
	/**
	 * Hottest liquid desiccant conditions from inlet at heat exchanger A.
	 * 
	 * @see Node
	 * @return Node instance with hottest liquid desiccant conditions from HX A.
	 */
	public Node getN_hAi(){
		return A.getSI();
	} 
	
	/**
	 * Adjusts the liquid desiccant conditions from inlet at pipe between heat
	 * exchanger A and heat exchanger G.
	 * 
	 * @see Node
	 * @param node Node instance with liquid desiccant conditions from inlet at Pipe AG.
	 * @throws OutOfRangeException The equations doesn't manage the actual properties range.
	 */
	public void setN_hAo(Node node) throws OutOfRangeException{
		AG.setNI(node);
	}
	
	/**
	 * Liquid desiccant conditions from inlet at pipe between heat exchanger A and
	 * heat exchanger G.
	 * 
	 * @see Node
	 * @return Node instance with liquid desiccant conditions from inlet at Pipe AG.
	 */
	public Node getN_hAo(){
		return AG.getNI();
	}
	
	public void setEp_A(double vare){
		A.setEffS(vare);
		A.setEffL(0.0);
	}
	
	public void setEp_K(double vare){
		K.setEffS(vare);
		K.setEffL(0.0);
	}

	public void setEp_G(double vare){
		G.setEffS(vare);
		G.setEffL(0.0);
	}
	
	public void setEp_D(double eS, double eL){
		D.setEffS(eS);
		D.setEffL(eL);
	}
	
	public void setEp_R(double eS, double eL){
		R.setEffS(eS);
		R.setEffL(eL);
	}
	
	/**
	 * Report the results
	 */
	public void report(){
		
	}
	
	/**
	 * Simulates the LDDS System
	 */
	public boolean run(){
		return true;
	}
	
	public void display(){
		
	}
	
}
